"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const text_manipulation_1 = require("./text-manipulation");
function applyFormat(normalizedContent, overrides = {}, separator = '', formatter = (s, i) => s) {
    return normalizedContent.map((each, index) => overrides[each.toLowerCase()] || formatter(each, index)).join(separator);
}
function normalize(identifier, removeDuplicates = true, overrides = {}) {
    if (!identifier || identifier.length === 0) {
        return [''];
    }
    return typeof identifier === 'string' ? normalize(text_manipulation_1.fixLeadingNumber(text_manipulation_1.deconstruct(identifier))) : removeDuplicates ? text_manipulation_1.removeSequentialDuplicates(identifier) : identifier;
}
class Style {
    static select(style, fallback) {
        switch (`${style}`.toLowerCase()) {
            case 'camelcase':
            case 'camel':
                return Style.camel;
            case 'pascalcase':
            case 'pascal':
                return Style.pascal;
            case 'snakecase':
            case 'snake':
                return Style.snake;
            case 'uppercase':
            case 'upper':
                return Style.upper;
            case 'kebabcase':
            case 'kebab':
                return Style.kebab;
            case 'spacecase':
            case 'space':
                return Style.space;
        }
        return fallback;
    }
    static kebab(identifier, removeDuplicates = true, overrides = {}) {
        return overrides[identifier] || applyFormat(normalize(identifier, removeDuplicates), overrides, '-');
    }
    static space(identifier, removeDuplicates = true, overrides = {}) {
        return overrides[identifier] || applyFormat(normalize(identifier, removeDuplicates), overrides, '');
    }
    static snake(identifier, removeDuplicates = true, overrides = {}) {
        return overrides[identifier] || applyFormat(normalize(identifier, removeDuplicates), overrides, '_');
    }
    static upper(identifier, removeDuplicates = true, overrides = {}) {
        return overrides[identifier] || applyFormat(normalize(identifier, removeDuplicates), overrides, '_', each => each.toUpperCase());
    }
    static pascal(identifier, removeDuplicates = true, overrides = {}) {
        return overrides[identifier] || applyFormat(normalize(identifier, removeDuplicates), overrides, '', each => each.capitalize());
    }
    static camel(identifier, removeDuplicates = true, overrides = {}) {
        return overrides[identifier] || applyFormat(normalize(identifier, removeDuplicates), overrides, '', (each, index) => index ? each.capitalize() : each.uncapitalize());
    }
}
exports.Style = Style;
//# sourceMappingURL=formatter.js.map